import binarysearchtree as BST

'''
Below are three simple example functions which investigate
a binary search tree processed in usageExample3 below.
   countLeaves
   countNodesWith1Child
   listNodesWith1Child
These functions are recursive, and to function properly,
they should be called with BST root as parameter.
See the code of usageExample3.
'''

def countLeaves( node ):
    """Returns the number of leaves in the tree
    which is rooted in node.
    """
    if node == None:
       return 0

    # Count a leaf.
    if node.left == None and node.right == None:
       return 1

    # Otherwise, total the results in the left and right
    # subtrees of the node.
    return countLeaves( node.left ) + countLeaves( node.right )




def countNodesWith1Child( node ):

    if node == None:
       return 0

    # Count nodes in left and right subtrees.
    totalCount  = countNodesWith1Child( node.left )
    totalCount += countNodesWith1Child( node.right )

    # Count the current node?
    if (node.left == None and node.right != None) or \
       (node.left != None and node.right == None):
       totalCount += 1

    return totalCount




def listNodesWith1Child( node, listOfNodes ):

    # No node, no action.
    if node == None:  return

    # Fill listOfNodes in the left subtree.
    listNodesWith1Child( node.left, listOfNodes )

    # Append the current node to listOfNodes?
    if (node.left == None and node.right != None) or \
       (node.left != None and node.right == None):
       listOfNodes.append( [node.key, node.value] )

    # Update listOfNodes in the right subtree.
    listNodesWith1Child( node.right, listOfNodes )



def usageExample3():

    IntroText = """
*****************************************************************
*   This is the THIRD introductory example.                ###  *
*   It demonstrates additional user defined functions     #   # *
*   which operate on a binary search tree.                    # *
*                                                           ##  *
*   Compare the output, step by step,                         # *
*   with the corresponding parts of the code.             #   # *
*   Feel free to experiment and modify the example.        ###  *
*****************************************************************
"""
    print( IntroText )

    # Create a binary search tree, before any further manipulation.
    t = BST.BinarySearchTree()

    # Build a random BST with 20 nodes and display it.
    t.randomBST( 17,   1, 30, 80, 99,  23581321 )
    t.display()

    # Use the function defined above to establish the number
    # of leaves and the number of nodes with single child in BST.
    print( 'BST has', countLeaves( t.root ), 'leaves.' )
    print( 'BST has', countNodesWith1Child( t.root ), 'nodes with 1 child.' )
    # print the nodes with single child
    listOfNodes = []
    listNodesWith1Child( t.root, listOfNodes )
    print( 'Those nodes are', listOfNodes )

    # See how the function result change after BST balance.
    t.balance()
    t.display()
    print( 'BST has', countLeaves( t.root ), 'leaves.' )
    print( 'BST has', countNodesWith1Child( t.root ), 'nodes with 1 child.' )

    listOfNodes = []
    listNodesWith1Child( t.root, listOfNodes )
    print( 'Those nodes are', listOfNodes )


# --------------------------------------------------------------------
#  RUN THE EXAMPLE
# --------------------------------------------------------------------

usageExample3()
